unit UtilComp;
{
 THeaderFocus - was derived from THeaderControl.
 Selection of single is possible. Selected section is highlighted with
 focuse rectangle.
 Property SelectedSection is used to get/set selection
 Additionally, two notification events were added: OnMouseEnter and
 OnMouseLeave - exposure internal notifications CM_MOUSEENTER and CM_MOUSELEAVE

 TSpeedBtColor was derived from TSpeedButton. Additional property Color is
 used to change brush color of the button.

 TListHScroll was derived from TListBox. Authomatical calculation of horizontal
 extension is added. Horizontal scroll bar appears authomatically with required
 range. Additional property Offset - left distance in pixels from default to draw
 text. It is used for owner-draw purposes.

 TComboSized was derived from TCombobox. It is resized authomatically when it
 is dropped down to show whole large string. Additional property Offset - left
 distance in pixels from  default to draw  text. It is used for owner-draw
 purposes.


FreeWare. No warranties. If one will distribute the source code, reference on
author (below) is required. If one will modify the source code, highlighting
(by any means) of modifications is required.

 Author: Sergei Trepalin, trep@trep.home.chg.ru
}

interface

uses
Windows, Messages, Classes, Controls, Forms, Graphics, StdCtrls,
  ExtCtrls, CommCtrl, Buttons, ComCtrls;

type
  THeaderFocus=class(THeaderControl)
  private
    FSelectedSection:THeaderSection;
    FOnMouseLeave,FOnMouseEnter:TNotifyEvent;
    procedure WMPaint(var Message:TWMPaint); message WM_PAINT;
    procedure CMMouseLeave(var Message:TMessage); message CM_MouseLeave;
    procedure CMMouseEnter(var Message:TMessage); message CM_MouseEnter;
    function  GetSelectedSection:integer;
    procedure SetSelectedSection(Value:integer);
  protected
    procedure SectionClick(Section: THeaderSection); override;
  published
    property SelectedSection:integer read GetSelectedSection write SetSelectedSection default -1;
    property OnMouseEnter:TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave:TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

  TSpeedBtColor = class(TSpeedButton)
  private
    { Private declarations }
    FColor:TColor;
  protected
    { Protected declarations }
     procedure Paint; override;
     procedure DrawButtonText(Canvas: TCanvas; const Caption: string;
       TextBounds: TRect; State: TButtonState);
     procedure SetColor(value:TColor);
 public
    { Public declarations }
  published
    { Published declarations }
    property Color:TColor read FColor write SetColor;
  end;

  TComboSized=class(TComboBox)
  private
    FWidthStore,FMaxWidth,FOffset:integer;
    FAcceptWidth:boolean;
    function CalculateWidth(const S:string):integer;
    procedure CNCommand(var Message:TWMCommand); message CN_COMMAND;
    procedure CBAddString(var Message:TMessage); message CB_ADDSTRING;
    procedure WMSize(var Message:TWMSize); message WM_SIZE;
    procedure CMFontChanged(var Message:TMessage); message CM_FONTCHANGED;
  public
    constructor Create(AOwner:TComponent); override;
    property Offset:integer read FOffset write FOffset;
  end;

  TListHScroll=class(TListBox)
  private
    FOffset:integer;
    function CalculateWidth(const S:string):integer; virtual;
    procedure LBAddString(var Message:TMessage); message LB_ADDSTRING;
    procedure CMFontChanged(var Message:TMessage); message CM_FONTCHANGED;
  public
    constructor Create(AOwner:TComponent); override;
    property Offset:integer read FOffset write FOffset;
  end;

  TComboBoxColorSel = class(TCustomComboBox)
  private
    procedure SetColor(value:TColor);
    function  GetColor:TColor;
    procedure DatabaseColor(const CName:string);
  protected
    { Protected declarations }
    procedure CreateWnd; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure   DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  published
    { Published declarations }
    property Color:TColor read GetColor write SetColor default clBlack;
    property Ctl3D;
    property DropDownCount;
    property Enabled;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
  end;


procedure Register;

implementation

procedure TSpeedBtColor.Paint;
var
  PaintRect: TRect;
begin
  if not Enabled and not (csDesigning in ComponentState) then
  begin
    FState := bsDisabled;
{    FDragging := False;}
  end
  else if FState = bsDisabled then FState:=bsUp;

  Canvas.Font := Self.Font;

  PaintRect := DrawButtonFace(Canvas, Rect(0, 0, Width, Height), 1, bsNew,
    False, FState in [bsDown, bsExclusive], False);

  Canvas.Brush.Color := Color;
  Dec(PaintRect.Right);
  Dec(PaintRect.Bottom);
  Canvas.FillRect(PaintRect);

  DrawButtonText(Canvas,Caption,PaintRect,FState);
end;

procedure TSpeedBtColor.SetColor(Value:TColor);
begin
  FColor:=Value;
  Invalidate;
end;

procedure TSpeedBtColor.DrawButtonText(Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; State: TButtonState);
begin
  with Canvas do begin
    Brush.Style:=bsClear;
    if State=bsDisabled then begin
      OffsetRect(TextBounds, 1, 1);
      Font.Color := clWhite;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, 0);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clDkGray;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, 0);
    end else
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds,
        DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;
end;

{---------------------------------------------------------------------------}
procedure THeaderFocus.WMPaint(var Message:TWMPaint);
var
  R:TRect;
begin
  inherited;
  if Assigned(FSelectedSection) and (Canvas.Handle<>0) and (GetSelectedSection>=0) then begin
    R.Left:=FSelectedSection.Left+4;
    R.Top:=4;
    R.Right:=FSelectedSection.Right-4;
    R.Bottom:=Height-4;
    Canvas.DrawFocusRect(R);
  end;
end;

procedure THeaderFocus.SectionClick(Section: THeaderSection);
begin
  FSelectedSection:=Section;
  inherited SectionClick(Section);
  invalidate;
end;

function THeaderFocus.GetSelectedSection:integer;
begin
  Result:=-1;
  if Assigned(FSelectedSection) and (Sections.Count>0) then begin
    repeat
    Result:=Result+1;
    until (FSelectedSection=Sections[Result]) or (Result=(Sections.Count-1));
    if FSelectedSection<>Sections[Result] then Result:=-1;
  end;
end;

procedure THeaderFocus.SetSelectedSection(Value:integer);
begin
  if (Value>=0) and (Value<Sections.Count) then if (Sections[Value]<>FSelectedSection) then begin
    FSelectedSection:=Sections[Value];
    Invalidate;
  end else if (Value=-1) and (Assigned(FSelectedSection)) then begin
    FSelectedSection:=nil;
    Invalidate;
  end;
end;

procedure THeaderFocus.CMMouseLeave(var Message:TMessage);
begin
  inherited;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure THeaderFocus.CMMouseEnter(var Message:TMessage);
begin
  inherited;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;
{--------------------------------------------------------------------------}
const
  BOXMAXLINE=32000;

constructor TComboSized.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FWidthStore:=Width;
  FMaxWidth:=0;
  FOffset:=0;
  FAcceptWidth:=True;
end;

procedure TComboSized.CNCommand(var Message:TWMCommand);
var
  N:integer;
begin
  case Message.NotifyCode of
    CBN_DROPDOWN: begin
      FAcceptWidth:=False;
      N:=FOffset+FMaxWidth+4;
      if Items.Count>DropDownCount then N:=N+GetSystemMetrics(SM_CXVSCROLL);
      if ClientWidth<N then ClientWidth:=N;
    end;
    CBN_CLOSEUP: begin
      if Width<>FWidthStore then Width:=FWidthStore;
      FAcceptWidth:=True;
    end;
  end;
  inherited;
end;

function TComboSized.CalculateWidth(const S:string):integer;
var
  DC:HDC;
  MCanvas:TCanvas;
begin
  DC:=0;
  Result:=0;
  MCanvas:=nil;
  try
    MCanvas:=TCanvas.Create;
    DC:=GetDC(0);
    if Assigned(MCanvas) and (DC<>0) then begin
      MCanvas.Handle:=DC;
      MCanvas.Font:=Font;
      Result:=MCanvas.TextWidth(S);
    end;
  finally
    if Assigned(MCanvas) then begin
      MCanvas.Handle:=0;
      MCanvas.Free;
    end;
    if DC<>0 then ReleaseDC(0,DC);
  end;
end;

procedure TComboSized.CBAddString(var Message:TMessage);
var
  S:string;
  N:integer;
begin
  S:=pchar(Message.lParam);
  N:=CalculateWidth(S);
  if N>FMaxWidth then FMaxWidth:=N;
  Message.Result:=0;
  if Items.Count<BOXMAXLINE then inherited;
end;

procedure TComboSized.WMSize(var Message:TWMSize);
begin
  inherited;
  if FAcceptWidth then FWidthStore:=Width;
end;

procedure TComboSized.CMFontChanged(var Message:TMessage);
var
  I,N:integer;
begin
  inherited;
  if Items.Count>0 then begin
    FMaxWidth:=0;
    for I:=0 to Items.Count-1 do begin
      N:=CalculateWidth(Items[I]);
      if N>FMaxWidth then FMaxWidth:=N;
    end;
  end;
end;
{-----------------------------------------------------------------}
constructor TListHScroll.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FOffset:=0;
end;

procedure TListHScroll.LBAddString(var Message:TMessage);
var
  S:string;
  N,M:integer;
begin
  S:=pchar(Message.lParam);
  N:=FOffset+CalculateWidth(S)+4;
  M:=Perform(LB_GETHORIZONTALEXTENT,0,0);
  if N>M then Perform(LB_SETHORIZONTALEXTENT,N,0);
  Message.Result:=0;
  if Items.Count<BOXMAXLINE then inherited;
end;

function TListHScroll.CalculateWidth(const S:string):integer;
var
  DC:HDC;
  MCanvas:TCanvas;
begin
  DC:=0;
  Result:=0;
  MCanvas:=nil;
  try
    MCanvas:=TCanvas.Create;
    DC:=GetDC(0);
    if Assigned(MCanvas) and (DC<>0) then begin
      MCanvas.Handle:=DC;
      MCanvas.Font:=Font;
      Result:=MCanvas.TextWidth(S);
    end;
  finally
    if Assigned(MCanvas) then begin
      MCanvas.Handle:=0;
      MCanvas.Free;
    end;
    if DC<>0 then ReleaseDC(0,DC);
  end;
end;

procedure TListHScroll.CMFontChanged(var Message:TMessage);
var
  I,N,M:integer;
begin
  inherited;
  if Items.Count>0 then begin
    M:=0;
    for I:=0 to Items.Count-1 do begin
      N:=CalculateWidth(Items[I]);
      if N>M then M:=N;
    end;
    if M>0 then Perform(LB_SETHORIZONTALEXTENT,M+4,0);
  end;
end;

{------------------------------------------------------------------------}
constructor TComboBoxColorSel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Font.Size := 8;
  Font.Style := Font.Style - [fsBold];
  Style := csOwnerDrawFixed;
  ItemHeight := 16;
  {Create a bitmap for drawing}
  DropDownCount:=4;
end;

procedure TComboBoxColorSel.CreateWnd;
begin
  inherited CreateWnd;
{  if (csDesigning in ComponentState) then Exit;}
  GetColorValues(DatabaseColor);
end;

procedure TComboBoxColorSel.DatabaseColor(const CName:string);
begin
  Items.AddObject(CName,pointer(StringtoColor(CName)));
end;

procedure TComboBoxColorSel.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  R:TRect;
  CS:TColor;
begin
  with Canvas do begin
    CS:=Brush.Color;
    FillRect(Rect);
    Brush.Color:=TColor(Items.Objects[Index]);
    Brush.Style:=bsSolid;
    R.Left:=Rect.Left+2;
    R.Right:=R.Left+24;
    R.Top:=Rect.Top+3;
    R.Bottom:=R.Top+ItemHeight-5;
    FillRect(R);
    Rectangle(R.Left-1,R.Top-1,R.Right+1,R.Bottom);
    Brush.Color:=CS;
    TextOut(Rect.Left+28, Rect.Top, Items[Index])
  end;
end;

procedure TComboBoxColorSel.SetColor(value:TColor);
var
  N:integer;
  Test:boolean;
begin
  N:=-1;
  repeat
    N:=N+1;
    Test:=Value=TColor(Items.Objects[N]);
  until Test or (N=(Items.Count-1));
  if Test then ItemIndex:=N;
end;

function TComboBoxColorSel.GetColor:TColor;
begin
  Result:=TColor(Items.Objects[ItemIndex]);
end;

procedure Register;
begin
  RegisterComponents('Interface', [TSpeedBtColor,THeaderFocus,TComboBoxColorSel,TComboSized,TListHScroll]);
end;


end.
